@@ -5,8 +5,8 @@ use std::convert::TryInto;
 use std::time::Duration;
 
 use serde::Serialize;
-
 use tokio::time::sleep;
+use tracing::instrument;
 
 use crate::Error;
 mod payload;
@@ -40,6 +40,7 @@ pub struct Client {
 
 impl Client {
 	// TODO:  Take Option<(&str, &str)> for auth
+	#[instrument(err, level = "info")]
 	pub fn new(host: &str, port: u16, tls: bool, auth: Option<(String, String)>) -> Result<Self, Error> {
 		let mut headers = reqwest::header::HeaderMap::new();
 		headers.insert(reqwest::header::USER_AGENT, "transmission-rs/0.1".try_into()?);
@@ -62,9 +63,12 @@ impl Client {
 	}
 
 	fn build_url(&self, path: impl AsRef<str>) -> String {
-		format!("{}{}", self.base_url, path.as_ref())
+		let mut url = self.base_url.clone();
+		url.push_str(path.as_ref());
+		url
 	}
 
+	#[instrument(err, level = "info", skip(self), fields(transmission.url = %self.base_url))]
 	pub async fn authenticate(&mut self) -> Result<(), Error> {
 		self.session_id = None;
 		let response = self.post("/transmission/rpc/")
@@ -78,6 +82,7 @@ impl Client {
 		Ok(())
 	}
 
+	#[instrument(level = "debug", skip(self, path), fields(transmission.url = %self.base_url, path = %path.as_ref()))]
 	pub fn post(&self, path: impl AsRef<str>) -> reqwest::RequestBuilder {
 		let url = self.build_url(path);
 		let mut request = self.http.post(&url);
@@ -92,6 +97,7 @@ impl Client {
 		request
 	}
 
+	#[instrument(err, level = "debug", skip(self, body), fields(transmission.url = %self.base_url))]
 	async fn rpc(&mut self, method: &str, tag: u8, body: impl Serialize) -> Result<reqwest::Response, Error> {
 		let request = Request::new(method, tag, body);
 		let mut error = None;
@@ -125,8 +131,9 @@ impl Client {
 		}
 	}
 
+	#[instrument(err, level = "info", skip(self), fields(transmission.url = %self.base_url))]
 	pub async fn list(&mut self) -> Result<Vec<Torrent>, Error> {
-		let field_list = FieldList::from_vec(vec![
+		let field_list = FieldList::new(&[
 			"error",
 			"errorString",
 			"eta",
@@ -147,16 +154,19 @@ impl Client {
 	}
 
 	// TODO:  Borrow key from value
+	#[instrument(err, level = "info", skip(self), fields(transmission.url = %self.base_url))]
 	pub async fn list_by_name(&mut self) -> Result<BTreeMap<String, Torrent>, Error> {
 		Ok(self.list().await?.into_iter().map(|v| (v.name.clone(), v)).collect::<BTreeMap<_, _>>())
 	}
 
+	#[instrument(err, level = "info", skip(self), fields(transmission.url = %self.base_url))]
 	pub async fn add_torrent_from_link(&mut self, url: &str) -> Result<AddTorrentResponse, Error> {
 		let response = self.rpc("torrent-add", 8, AddTorrent{filename: Some(url), metainfo: None}).await?.error_for_status()?.text().await?;
 		let response: Response<AddTorrentResponse> = serde_json::from_str(&response)?;
 		Ok(response.arguments)
 	}
 
+	#[instrument(err, level = "info", skip(self), fields(transmission.url = %self.base_url))]
 	pub async fn add_torrent(&mut self, url: &str) -> Result<AddTorrentResponse, Error> {
 		let response = reqwest::get(url).await?.bytes().await?;
 		let torrent_data = base64::encode(&response);
@@ -165,6 +175,7 @@ impl Client {
 		Ok(response.arguments)
 	}
 
+	#[instrument(err, level = "info", skip(self), fields(transmission.url = %self.base_url))]
 	pub async fn get_files_by_id(&mut self, id: u16) -> Result<Vec<TorrentFile>, Error> {
 		let response: Response<GetTorrentFilesResponse> = self.rpc("torrent-get", 3, GetTorrentFilesRequest::new(id)).await?.error_for_status()?.json().await?;
 		if(response.arguments.torrents.is_empty()) {
