@@ -14,9 +14,9 @@ pub struct FieldList {
 }
 
 impl FieldList {
-	pub fn from_vec(vec: Vec<impl ToString>) -> Self {
+	pub fn new(fields: &[impl ToString]) -> Self {
 		Self{
-			fields: vec.iter().map(|s| s.to_string()).collect()
+			fields: fields.iter().map(|s| s.to_string()).collect()
 		}
 	}
 }
